# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.test import TestCase

from django.contrib.auth.models import User
from django.core.urlresolvers import reverse

from unsubscribe.mail import UnsubscribableEmailMessage
from unsubscribe.utils import get_token_for_user


class UnsubscribeTest(TestCase):

    def setUp(self):
        self.user = User(
            username='unsubscribe-testuser', email='testuser@sdaf.com')
        self.user.save()

    def test_list_unsubscribe_headers(self):
        msg = UnsubscribableEmailMessage(self.user, "Test Message", "Body",
                                         from_email="test@testserver.com", to=["testemail@somewhereelse.com"])
        msg.send()

        from django.core.mail import outbox
        self.assertEquals(len(outbox), 1)
        self.assertTrue('List-Unsubscribe' in outbox[0].extra_headers.keys())
        self.assertEquals(
            outbox[0].extra_headers['List-Unsubscribe'], '<' + msg.unsubscribe_url + '>')

    def test_list_unsubscribe_view(self):
        closure_test = [0]

        def test_callback(sender, user, **kwargs):
            closure_test[0] = 1

        from unsubscribe.signals import user_unsubscribed
        user_unsubscribed.connect(test_callback)

        from django.test.client import Client
        c = Client()
        url = reverse('unsubscribe_unsubscribe',
                      args=(self.user.pk, get_token_for_user(self.user)))
        c.get(url)
        self.assertTrue(closure_test[0])



# -*- coding: utf-8 -*-
from django.shortcuts import get_object_or_404, render_to_response
from django.http import Http404
from django.template import RequestContext

from .compat import get_user_model
from .utils import get_token_for_user
from .signals import user_unsubscribed


def unsubscribe(request, user_id, token, template='unsubscribe/unsubscribe.html',
                extra_context=None):
    """
    Checks the user token and fires `unsubscribe.signals.user_unsubscribed` and
    returns unsubscribe/unsubscribe.html with extra_context, which could include
    callables and `unsubscribe_user`, which is the user that is unsibscribing.
    """
    User = get_user_model()
    user = get_object_or_404(User, pk=user_id)
    if not token == get_token_for_user(user):
        raise Http404

    context = RequestContext(request)

    if extra_context is None:
        extra_context = {}
    for key, value in extra_context.items():
        context[key] = callable(value) and value() or value

    user_unsubscribed.send(sender=User, user=user)

    return render_to_response(template, {'unsubscribe_user': user}, context_instance=context)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from .compat import patterns, url

urlpatterns = patterns('unsubscribe.views',  # noqa
    url(r'^(?P<user_id>\d*)-(?P<token>.*)/$',
        'unsubscribe', name="unsubscribe_unsubscribe"),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
# Django settings for testproject project.

import os
from os.path import join

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

SECRET_KEY = 'verysecret'

DEBUG = True
TEMPLATE_DEBUG = DEBUG
DEBUG_PROPAGATE_EXCEPTIONS = True

ALLOWED_HOSTS = ['*']

# Application definition
INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    'unsubscribe',
    'unsubscribe.tests',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'unsubscribe.urls'


# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME':  'db.sqlite3',
    }
}

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    # 'django.template.loaders.eggs.Loader',
)

TEMPLATE_DIRS = (
    'templates',
    join(BASE_DIR, 'templates')
)

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True
USE_TZ = True

SITE_ID = 1

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

STATIC_URL = '/static/'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.core.urlresolvers import reverse
from django.contrib.sites.models import Site

from .utils import get_token_for_user


class UnsubscribableEmailMessage(EmailMultiAlternatives):

    """
    A simple wrapper around EmailMultiAlternatives that requires a User
    object and appends a `List-Unsubscribe` header to the email message
    """

    def __init__(self, user, subject='', body='', from_email=None, to=None,
                 bcc=None, connection=None, attachments=None,
                 headers=None, alternatives=None):
        unsub_headers = headers or {}
        unsub_url = reverse('unsubscribe_unsubscribe',
                            args=[user.pk, get_token_for_user(user)])

        # TODO fix scheme not to be hard coded.
        protocol = 'http'
        site_url = '%s://%s' % (protocol, Site.objects.get_current().domain)
        unsub_headers['List-Unsubscribe'] = '<%s%s>' % (site_url, unsub_url)
        super(UnsubscribableEmailMessage, self).__init__(subject=subject,
                                                         body=body, from_email=from_email, to=to, bcc=bcc,
                                                         connection=connection, attachments=attachments,
                                                         headers=unsub_headers, alternatives=alternatives)

    def _get_unsubscribe_url(self):
        return self.extra_headers['List-Unsubscribe'][1:-1]

    def _set_unsubscribe_url(self, value):
        self.extra_headers['List-Unsubscribe'] = "<%s>" % value

    unsubscribe_url = property(_get_unsubscribe_url, _set_unsubscribe_url)

    def render_message(self, template, context=None):
        """
        A wrapper around render_to_string which feeds the template the
        additional context, `unsubscribe_url` which is the url for the user to
        unsubscribe
        """
        if not context:
            context = {}
        context['unsubscribe_url'] = self.unsubscribe_url
        return render_to_string(template, context)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import hashlib
from django.conf import settings


def get_token_for_user(user):
    # TODO: use USERNAME_FIELD instead
    user_id = user.email.encode('utf-8')
    secret = settings.SECRET_KEY.encode('utf-8')
    return hashlib.md5(user_id + secret).hexdigest()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.match("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')
version = get_version('unsubscribe')

if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()


setup(
    name='django-unsubscribe',
    version=version,
    url='https://github.com/theskumar/django-unsubscribe',
    description='Add unsubscribe link to your emails painlessly.',
    long_description=readme + '\n\n' + history,
    author='Saurabh Kumar',
    author_email='me+github@saurabh-kumar.com',
    packages=get_packages('unsubscribe'),
    package_data=get_package_data('unsubscribe'),
    keywords=['email', 'django', 'unsubscribe'],
    test_suite='unsubscribe.runtests.runtests.main',
    install_requires=[],
    zip_safe=False,
    classifiers=[
        # As from https://pypi.python.org/pypi?%3Aaction=list_classifiers
        # 'Development Status :: 1 - Planning',
        # 'Development Status :: 2 - Pre-Alpha',
        # 'Development Status :: 3 - Alpha',
        'Development Status :: 4 - Beta',
        # 'Development Status :: 5 - Production/Stable',
        # 'Development Status :: 6 - Mature',
        # 'Development Status :: 7 - Inactive',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        # 'Programming Language :: Python :: 2.3',
        # 'Programming Language :: Python :: 2.4',
        # 'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        # 'Programming Language :: Python :: 3.0',
        # 'Programming Language :: Python :: 3.1',
        # 'Programming Language :: Python :: 3.2',
        # 'Programming Language :: Python :: 3.3',
        'Intended Audience :: Developers',
        # 'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        # 'Topic :: System :: Systems Administration',
        'Topic :: Utilities',
        'Environment :: Web Environment',
        'Framework :: Django',
    ]
)

# (*) Please direct queries to the discussion group, rather than to me directly
#     Doing so helps ensure your question is helpful to other users.
#     Queries directly to my email are likely to receive a canned response.
#
#     Many thanks for your understanding.



"""
The `compat` module provides support for backwards compatibility with older
versions of django/python, and compatibility wrappers around optional packages.
"""

# flake8: noqa
from __future__ import unicode_literals

import django

# location of patterns, url, include changes in 1.4 onwards
try:
    from django.conf.urls import patterns, url, include
except ImportError:
    from django.conf.urls.defaults import patterns, url, include

# get_user_model does not exist in 1.4
try:
    from django.contrib.auth import get_user_model
except ImportError:
    from django.contrib.auth.models import User
    get_user_model = lambda: User



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.test import TestCase


class UtilsTests(TestCase):
    """docstring for UtilsTests"""

    def setUp(self):
        self.username = 'theskumar'
        self.email = 'theskumar@example.com'

    def test_foo(self):
        self.assertEqual('foo', "foo")
